import 'dart:async';

import 'package:aitrainer_app/model/cache.dart';
import 'package:aitrainer_app/model/exercise_device.dart';
import 'package:aitrainer_app/repository/customer_exercise_device_repository.dart';
import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';

part 'customer_exercise_device_event.dart';
part 'customer_exercise_device_state.dart';

class CustomerExerciseDeviceBloc extends Bloc<CustomerExerciseDeviceEvent, CustomerExerciseDeviceState> {
  final CustomerExerciseDeviceRepository repository;
  final List<ExerciseDevice> devices;

  CustomerExerciseDeviceBloc({this.repository, this.devices}) : super(CustomerExerciseDeviceInitial()) {
    if (repository.getDevices().isEmpty) {
      repository.setDevices(Cache().getCustomerDevices());
    }
  }

  @override
  Stream<CustomerExerciseDeviceState> mapEventToState(
    CustomerExerciseDeviceEvent event,
  ) async* {
    try {
      if (event is CustomerExerciseDeviceLoad) {
        yield CustomerExerciseDeviceLoading();
        yield CustomerExerciseDeviceReady();
      } else if (event is CustomerExerciseDeviceAdd) {
        yield CustomerExerciseDeviceLoading();
        //await Future.delayed(const Duration(seconds: 2), () => "2");
        await repository.addDevice(event.device);
        Cache().initBadges();
        yield CustomerExerciseDeviceReady();
      } else if (event is CustomerExerciseDeviceRemove) {
        yield CustomerExerciseDeviceLoading();
        await repository.removeDevice(event.device);
        Cache().initBadges();
        yield CustomerExerciseDeviceReady();
      }
    } on Exception catch (ex) {
      yield CustomerExerciseDeviceError(message: ex.toString());
    }
  }

  bool hasCustomerDevice(int exerciseDeviceId) {
    return repository.hasDevice(exerciseDeviceId);
  }
}
